/* hdrs/net.h */

/* Connection Flags */
#define C_CONNECTED	BV0	// Player is connected to the main game
#define C_LINEMODE	BV1	// Client supports Telnet EOR protocol
#define C_REMOTE	BV2	// Remote connection under concentrator
#define C_OUTBOUND	BV3	// This is an outbound connection
#define C_LOGIN		BV4	// User login initialization
#define C_WAITCONNECT	BV5	// Connect to game when database finishes load
#define C_RECONNECT	BV6	// Game will reconnect player at next chance
#define C_LOGOFF	BV7	// Game will log player off at next chance
#define C_DESTROY	BV8	// Game will destroy player character at logout
#define C_PARENT	BV9	// Concentrator parent, used with Remote
#define C_OUTPUTOFF	BV10	// Output temporarily saved in buffers
#define C_INPUTOFF	BV11	// Input not received or processed
#define C_SBAROFF	BV12	// No status bar updates
#define C_RESOLVER	BV13	// Awaiting response from DNS query
#define C_SOCKET	BV14	// FD is a socket listening for connections
#define C_DNS		BV15	// FD is a socket receiving DNS messages
#define C_SERVICE	BV16	// Currently supporting an I/O service
#define C_SUPPRESS	BV17	// Suppress Login/Logout messages

/* flush_io() parameters */
#define INPUT_STREAM	0	// Input buffer containing commands from client
#define OUTPUT_STREAM	1	// Output on queue to be sent to fd
#define SERVER_STREAM	2	// High-priority server output; can't turn off

#define ALL_IO		-1	// Flushes all I/O streams
#define PENDING_INPUT	-2	// Flushes only unread input from socket


   /* Battle Skill Summary */

#define FS_HITS		0	// Number of successful attacks
#define FS_MISSES	1	// Target misses
#define FS_CRITS	2	// Terrific shots
#define FS_PARRIES	3	// Shots blocked
#define FS_RIPOSTES	4	// Rebound attacks

#define FS_STUNS	5	// Stunned maneuvers
#define FS_DODGES	6	// Enemy attack dodges
#define FS_FUMBLES	7	// Number of tumbles
#define FS_KILLS	8	// Enemies slain
#define FS_DEATHS	9	// Times you've been killed

/* these values are set at login, compared to current at 'summary' */
#define FS_EXPGAIN	10	// Experience gained
#define FS_GPGAIN	11	// Gold value difference
#define FS_STEPS	12	// Initial number of steps

/* regular values */
#define FS_FOOD		13	// Amount of food eaten
#define FS_DMGGIVEN	14	// Total damage inflicted

#define FS_DMGPROT	15	// Damage with protection
#define FS_ELEMPROT	16	// Values 16-25 are elemental-specific

#define FS_DMGTAKEN	26	// Damage actually taken by attacks
#define FS_ELEMTAKEN	27	// Values 27-36 are elemental-specific

/* maximum fight summary value +1 */
#define FIGHT_SUMMARY	37


   /* netstat macros */

#define NS_FILES	0	// Number of file descriptors in use
#define NS_OUTPUT	1	// Server output
#define NS_INPUT	2	// Server input
#define NS_EXEC		3	// Reboots (program executions)
#define NS_CRASHES	4	// Number of segmentation faults produced
#define NS_DBSAVE	5	// Number of database saves
#define NS_LOGINS	6	// Top connection-id established
#define NS_CONNECTS	7	// Successful player connects
#define NS_FAILED	8	// Failed connects
#define NS_NEWCR	9	// Newly created players
#define NS_GUESTS	10	// Number of guest players created
#define NS_NCMDS	11	// Number of commands
#define NS_NPCMDS	12	// Player commands executed
#define NS_NQCMDS	13	// Queue commands executed
#define NS_NFCMDS	14	// Failed commands
#define NS_QUEUE	15	// Queue process calls
#define NS_OVERRUNS	16	// Queue time-slice overruns
#define NS_MAIL		17	// Mail messages sent
#define NS_EMAIL	18	// Internet Email sent
#define NS_ERECV	19	// Email received (SMTP Mode)
#define NS_RESOLV	20	// Nameserver lookups sent
#define NS_QUERY	21	// DNS hostname queries resolved

#define NS_ATKS		22	// Total attacks
#define NS_GOLD		23	// World summation gold gained from monsters
#define NS_TCHEST	24	// Treasure chests obtained from monsters
#define NS_EXP		25	// World summation experience gained
#define NS_LEVEL	26	// Levelups achieved
#define NS_TECHPTS	27	// World summation technique points gained
#define NS_DMONSTER	28	// Monster defeats
#define NS_DPLAYER	29	// Player kills
#define NS_ALMANAC	30	// Almanac updates
#define NS_CASTROD	31	// Number of fishing rods cast
#define NS_FISH		32	// Number of fish caught
#define NS_PLANTS	33	// Number of plants picked

/* maximum netstat value +1 */
#define NET_STATS	34

/* Note: When changing netstats, please change the corresponding value in
   function(netstat) in prog/eval.c */

extern int nstat[NET_STATS];


  /* dbstat macros */

#define DB_EXEC		0	// Number of program executions
#define DB_DUMPS	1	// Number of database saves
#define DB_LOGINS	2	// Logins established
#define DB_CONNECTS	3	// Successful connects
#define DB_NEWCR	4	// New players created
#define DB_MAIL		5	// +Mail messages sent

#define DB_STATS	6

extern int dbstat[DB_STATS];


  /* terminfo entries */

#define TERM_ANSI	BV0	// 0=strip all ansi escape codes from output
#define TERM_HIGHFONT	BV1	// 0=strip 8-bit font down to 7-bit
#define TERM_GRAPHICS	BV2	// 1=use special graphic icons in display
#define TERM_BEEPS	BV3	// 1=beeps are enabled (default)
#define TERM_SOUND	BV4	// 1=send ansi sound/music codes
#define TERM_REGINFO	BV5	// 1=display email & rlname on +finger
#define TERM_WORDWRAP	BV6	// 1=perfect wordwrap mode for telnet clients
  /* add TERM_TERSE, TERM_AUTOSPLIT (split gold gained with group members),
         TERM_ASSIST (assist group members in combat) */

extern struct term_config {
  char *name;
  int flag;
} term_config[];

/* Socket types */

#define SOCKET_GAME	0	// Everyone can log into this port
#define SOCKET_ADMIN	1	// Only admin can log in here
#define SOCKET_HTTP	2	// Web server access only
#define SOCKET_MESG	3	// Display message and disconnect user

/* Time format displays */

#define TMA		0	// All time values: 17d, 16h, 45m
#define TML		1	// Online time, Long format: 17d 16:45m
#define TMS		2	// Idle time, Short format: 2w
#define TMF		3	// Full format: 2 weeks

/* Special connection states */

#define GAME_STATE	255	// Normal command processing while connected

#define ENV_SIZE	4	// Environment variables for connection state

struct io_queue {
  struct io_queue *next;
  char text[0];
};

/* Network Connection Structure */

typedef struct descriptor_data {
  struct descriptor_data *next;

  /* main descriptor data */
  int descriptor;			// File descriptor
  int flags;				// Connection flags
  int concid;				// Connection ID#
  dbref player;				// Enacting player dbref#

  /* concentrator info */
  struct descriptor_data *parent;	// For remote concentrators
  long login_time;			// Time of game login
  long last_time;			// Time last command was entered
  long connected_at;			// Time the game session begins

  /* login server position */
  char *env[ENV_SIZE];			// Environment variables
  int timer;				// Login initialization timer
  int state;				// Connection state
  unsigned int socket:16;		// Port to which connection originated
  unsigned int type:8;			// Type of port (SOCKET_GAME, etc..)

  /* input/output strings */
  struct io_queue *io[3], *io_last[3];	// Text queues: in/out/server
  char *raw_input;			// Current command input
  int output_size;			// Current pending output amount
  int cmds;				// Total commands typed in
  int input;				// Total input to game driver
  int output;				// Total output to player

  /* internet address */
  char *addr;				// Source hostname
  int ipaddr;				// Source IP Address
  unsigned int port:16;			// Port on remote server

  /* terminal settings */
  unsigned int rows:16;			// Number of rows on console
  unsigned int cols:16;			// Number of columns on console
  unsigned int term:8;			// Terminal type

  /* session summary */
  int summary[FIGHT_SUMMARY];
} DESC;

extern DESC *descriptor_list;


/* Server Command Structure */

struct srvcmd {
  int state;
  char *title;
  int (*fun)(struct descriptor_data *, char *);
};

extern struct srvcmd server_command[];

/* Macros to send Telnet IACs */

#define PROMPT(d) ((d->flags & C_LINEMODE)?"\377\357":"")
#define ECHO_OFF "\377\373\001"
#define ECHO_ON "\377\374\001"

extern char *services[];

extern long mud_up_time;
extern long last_reboot_time;
extern long open_files;

extern int halt_dump;
extern long dump_interval;
extern int epoch_fail;
extern int cpu_percent;
extern int cmdav[2];
extern double cmdfp[2];

extern int shutdown_flag;
extern int disable_startups;
extern int game_online;
extern int resolv;

extern long now;
extern quad time_slice;
